/*!
 * \file libmain.hpp
 * \brief This file is used to common works.<br>
 *        e.g. initialization, finalization, etc...
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#ifndef LIBMAIN_HPP
#define LIBMAIN_HPP

#include <jni.h>
#include <jvmti.h>

#include "util.hpp"
#include "timer.hpp"
#include "signalManager.hpp"
#include "jvmInfo.hpp"

/* Agent_OnLoad/JNICALL Agent_OnAttach return code. */

/*!
 * \brief Process is successed.
 */
#define SUCCESS                          0x00

/*!
 * \brief Failure set capabilities.
 */
#define CAPABILITIES_SETTING_FAILED      0x01
/*!
 * \brief Failure set callbacks.
 */
#define CALLBACKS_SETTING_FAILED         0x02
/*!
 * \brief Failure initialize container.
 */
#define CLASSCONTAINER_INITIALIZE_FAILED 0x03
/*!
 * \brief Failure initialize agent thread.
 */
#define AGENT_THREAD_INITIALIZE_FAILED   0x04
/*!
 * \brief Failure create monitor.
 */
#define MONITOR_CREATION_FAILED          0x05
/*!
 * \brief Failure get environment.
 */
#define GET_ENVIRONMENT_FAILED           0x06
/*!
 * \brief Failure get low level information.<br>
 *        E.g. symbol, VMStructs, etc..
 */
#define GET_LOW_LEVEL_INFO_FAILED        0x07

/*!
 * \brief Event enable switch macro.
 */
#define SWITCH_EVENT_NOTIFICATION(jvmti, event, mode) \
    if (isError((jvmti), \
        (jvmti)->SetEventNotificationMode((mode), (event), NULL))) { \
        return CALLBACKS_SETTING_FAILED; \
    }

/*!
 * \brief This macro is notification catch signal to signal watcher thread.
 */
#define NOTIFY_CATCH_SIGNAL intervalSigTimer->notify();

/* Define export function for calling from external. */
extern "C"{
    /*!
     * \brief Agent attach entry points.
     * \param vm       [in] JavaVM object.
     * \param options  [in] Commandline arguments.
     * \param reserved [in] Reserved.
     * \return Attach initialization result code.
     */
    JNIEXPORT jint JNICALL Agent_OnLoad(JavaVM *vm, char *options,
        void *reserved);
    
    /*!
     * \brief Common agent unload entry points.
     * \param vm [in] JavaVM object.
     */
    JNIEXPORT void JNICALL Agent_OnUnload(JavaVM *vm);
    
    /*!
     * \brief Ondemand attach's entry points.
     * \param vm       [in] JavaVM object.
     * \param options  [in] Commandline arguments.
     * \param reserved [in] Reserved.
     * \return Ondemand-attach initialization result code.
     */
    JNIEXPORT jint JNICALL Agent_OnAttach(JavaVM *vm, char *options,
        void *reserved);
};

/*!
 * \brief Abort JVM by force on illegal status.
 * \param jvmti    [in] JVMTI environment object.
 * \param env      [in] JNI environment object.
 * \param causeMsg [in] Message about cause of aborting JVM.
 * \warning This function is always no return.
 */
void forcedAbortJVM(jvmtiEnv *jvmti, JNIEnv *env, const char *causeMsg);

/* Agent components. */

extern TJvmInfo *jvmInfo;
extern TSignalManager *reloadSigMngr;
extern TTimer *intervalSigTimer;

#endif // LIBMAIN_HPP
